 /************************************************************************ 
  *                                                                      *
  *    NList 1.2.16 Copyright (C) 1997 Erich Menge (erich@minn.net)      *
  *		Enhanced by Gabor Paller (paller@javasite.bme.hu)	 *
  *			(C) 1998-2003					 *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
  * it under the terms of the GNU General Public License as published by *
  * the Free Software Foundation; either version 2 of the License, or    *
  * (at your option) any later version.                                  *
  *                                                                      *
  * This program is distributed in the hope that it will be useful,      *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
  * GNU General Public License for more details.                         *
  *                                                                      *
  * You should have received a copy of the GNU General Public License    *
  * along with this program; if not, write to the Free Software          *
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
  *                                                                      *
  ************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <setjmp.h>
#include <time.h>
#include <sys/timeb.h>
#include <errno.h>
#define MAIN
#include "config.h"
#include "decoder.h"
#include	<stdarg.h>
#if defined SOLARIS
#include	<sys/varargs.h>
#endif
#define		N_EXTERN
#include	"commonvars.h"

union {
  time_t		time;
  unsigned long		ltime;
  unsigned char		ctime[8];
} tc_union;

/* Updates the confirmation file. Filters through the file and removes
   timeouted entries. Returns true if the parameter address was found
   and the passcode is correct. If the address matches (whether or not
   the passcode is correct), the entry is removed. Confirmation
   file structure:
   <e-mail address> <passcode> <timestamp>  */
bool UpdateConfirmationFile( char address[], char passcode[] ) {
  char	conffilename[N_BUFSIZE];
  char	backupfilename[N_BUFSIZE];
  char	buf[N_BUFSIZE],buf2[N_BUFSIZE];
  char	*p1,*p2,*p3;
  FILE	*conf,*backup;
  bool	EntryRemoved = FALSE;
  unsigned long	timeout,timelimit,u;

/* Calculating the limit time before which confirmation request entries
   will be timed out */
  timeout = ((unsigned long)SubsConfTimeout)*60L*60L;
  tc_union.time = time( NULL );
  timelimit = tc_union.ltime - timeout;
  
  sprintf( conffilename, "%s/%s.scdata", PATH, list_name );
  conf = fopen( conffilename, "r");
  if( conf != NULL ) {
    sprintf( backupfilename, "%s/%s.scdata_bak", PATH, list_name );
    backup = file_open( backupfilename, "w");
    while( fgets( buf,N_BUFSIZE-1,conf ) != NULL ) {
      strcpy( buf2,buf );
      p1 = strtok( buf2," " );
      if( p1 == NULL )
      	continue;
      p2 = strtok( NULL," " );
      if( p2 == NULL )
      	continue;
      p3 = strtok( NULL," " );
      if( p3 == NULL )
      		continue;
      if( sscanf( p3,"%lu",&u ) < 1 )
      	continue;
      if( u < timelimit ) {	/* Delete the entry if timed out */
        Log( 4,1,"Subscription confirmation entry from %s with passcode %s timed out",
        	p1,p2 );
      	continue;
      }
      if( strcasecmp( p1,address ) == 0 ) {
        if( ( passcode != NULL ) && ( strcmp( passcode,p2 ) == 0 ) ) 
        		EntryRemoved = TRUE;
        continue;
      }
      fprintf( backup,"%s %s %lu\n", p1, p2, u );
    }
    fclose( backup );
    fclose( conf );
/* Replacing the old confirmation file with the updated one */
    unlink( conffilename );
    rename( backupfilename, conffilename ); 
  }
  return EntryRemoved;
}

/* Add the given address to the confirmation file. Returns the password
   string associated with the confirmation */
char *AddConfirmationFile( char address[] ) {
  char	conffilename[N_BUFSIZE];
  FILE	*conf;
  static char	retval[32];
  unsigned char code[8];
  int	i;
  char	*p;

/* Create the passcode */
  tc_union.time = time( NULL );
  for( i = 0 ; i < 8 ; ++i )
  	code[i] = tc_union.ctime[i];
  for( i = 0 ; i < 8 ; ++i ) {
    if( i >= strlen( address ) )
    	break;
    code[i] ^= address[i];
  }
  p = retval;
  for( i = 0 ; i < 8 ; ++i ) {
    sprintf( p,"%02X",code[i] );
    p += 2;
  }
  sprintf( conffilename, "%s/%s.scdata", PATH, list_name );
  conf = file_open( conffilename,"a" );
  fprintf( conf,"%s %s %lu\n",address,retval,tc_union.ltime );
  fclose( conf );
  return retval;
}

/* Sends the obligatory header lines */
void send_list_headers( char *addressee ) {
/*This header is always sent to detect mail loops */
  sprintf( work,"X-Sender: nlist %s\n",NLIST_VERSION );
  write_smtp_line( work );
/* This header conforms with sendmail */
  sprintf( work,"Auto-Submitted: auto-generated (mail list message)\n" );
  write_smtp_line( work );
  if( *to_address ) {
    sprintf( work,"To: %s\n",
	    subst_lname( to_address,addr,addressee,list_address ) );
    write_smtp_line( work );
  }
  if( *replyto ) {
    sprintf( work,"Reply-To: %s\n",
	    subst_lname( replyto,addr,addressee,list_address ) );
    write_smtp_line( work );
  }
  if( *errors_to ) {
    sprintf( work,"Errors-To: %s\n",
	    subst_lname( errors_to,addr,addressee,list_address ) );
    write_smtp_line( work );
  }
}

/* Sends back a reply letter. Gets the destination address and the
   name of the template file */
void send_reply( char dest[], char templatename[], char reply_subject[] ) {
  char	c,*p1,*p2,*p3,*p4;
  int	len,numread;
  FILE	*template,*tempfile;
  char	buf[N_BUFSIZE];

  if( *list_name == 0 )
      p3 = "unknown_list";
  else
      p3 = list_name;
  sprintf( fname,"%s/%s.%s",PATH,list_name,templatename );
  if( ( template = fopen( fname,"r" ) ) == NULL )
	return;		/* Quit in silence if file doesn't exist */
  if( open_smtp_session( smtp_server,list_address,addr ) )
	ehandle( 1,"SMTP error for %s\n",dest );
  sprintf(work, "Subject: " );
  if( *list_title != 0 ) {
    strcat( work,list_title );
    strcat( work," " );
  }
  strcat( work," Re: " );
  if( reply_subject != NULL )
    strcat( work,reply_subject );
  strcat( work,"\n" );
  write_smtp_line( work );
  send_list_headers( addr );
/* Now comes the message body */
  write_smtp_line( "\n" );

/* Sending out text in pieces */
  while( fgets( work,N_BUFSIZE,template ) != NULL ) {
    p1 = work;
    p2 = buf;
    while( c = *(p1++) ) {
      if( c == '[' ) {
	if( strncmp( p1,"listname]",9 ) == 0 ) {
	  strcpy( p2,p3 );
	  p1 += 9;
	  p2 += strlen( p3 );
	  continue;
	} else
	if( strncmp( p1,"sender]",7 ) == 0 ) {
	  p1 += 7;
	  strcpy( p2,addr );
	  p2 += strlen( addr );
	  continue;
	}
      }
      *(p2++) = c;
    }
    *p2 = 0;
    write_smtp_line( buf );
  }
  fclose( template );
/* Appending the original message now */
  sprintf( buf,"------------------------------------------------------\n" );
  write_smtp_line( buf );
  tempfile = file_open( TempFileName,"r" );
  while( fgets( buf,N_BUFSIZE-1,tempfile ) != 0 )
	write_smtp_line( buf );
  fclose( tempfile );
  close_smtp_session();
}

/* Handles subscribe requests. Gets a pointer to the subject
   header pointing after "subscribe" word */
void handle_subscribe( char *adr2, bool ignore_subsconf ) {
  FILE *deny;
  FILE *approve;
  FILE *member;
  char **members;
  int pass=1;
  char *aof;
  
  if( adr2 != NULL ) {
    adr2 = adr2 + 9;
    adr2 = strtok( adr2," \n\t" );
  }
  if( ( adr2 == NULL ) || ( *adr2 == 0 ) ) {
    if( *list_name == 0 || control_mode )	/* List name not yet set */
	longjmp( finish,BAD_SUBSCRIBE );
  } else
	strcpy( list_name,adr2 );
  read_config( list_name,0 );
  Log( 2,1,"subscription request on list %s for %s",list_name,addr );
/* Quit if the list is closed for subscription */
  if( !subscribe ) {
    Log( 2,1,"list %s is closed for subscription",list_name );
    longjmp( finish, LIST_DISABLED );
  }
/* If confirmation is requested and this flag shouldn't be ignored,
   send confirmation request to the sender instead of effectively
   do the subscription */
  if( SubsConfirmation && !ignore_subsconf ) {
/* Update the confirmation file while removing previous subscription
   confirmation entry belonging to the sender */
    UpdateConfirmationFile( addr, NULL );
/* Generate the subject for the confirmation request letter */
    sprintf( SubsConfID, "confirm %s",AddConfirmationFile( addr ) );
    longjmp( finish, SEND_CONF_MESSAGE );
  }
  sprintf(fname, "%s/%s.deny", PATH, list_name );
  deny=fopen( fname, "r");
  sprintf(fname, "%s/%s.approve", PATH, list_name );
  approve=fopen( fname, "r");
  sprintf(fname, "%s/%s.members", PATH, list_name);
  member = file_open( fname,"r+" );
/* Quit if the member is already on the list */
  while(fscanf(member, "%s", work)!=EOF)
	if(!strcasecmp(addr, work)) {
	  Log( 2,1,"address %s is already on list %s",
	              addr,list_name );
	  longjmp( finish, ALREADY_ON_LIST );
	}
  pass=1;
  if(deny) {
     while(((aof=getline(deny))!=NULL) && pass==1) {
       if(((strstr(aof, addr))!=NULL) ||
	   (!strncmp(aof, "*\0", 2)) ||
	   (!strncmp(aof, "ALL\0", 4))) {
		pass=0;
                break;
       }
     }
  }
  if(approve) {
     while((aof=getline(approve))!=NULL && pass==0) {
	if((strstr(aof, addr))!=NULL)
		pass=1;
     }
  }
/* Quit if subscription from this address is not allowed */        
  if (pass==0) {
    Log( 2,1,"subscription to list %s from %s denied",list_name,
		addr );
    longjmp( finish, NOT_ALLOWED );
  }
/* Adding the user to the list */                
  fprintf(member, "%s\n", addr);
  fclose( member );
  longjmp( finish,SEND_SUBS_MESSAGE );  
}

/* Handles unsubscribe messages */
void handle_unsubscribe( char *adr2, char *addr, int final_status ) {
  FILE *member;
  int len;
  int numlines=0;
  int x=0;
  int	success = 0;
  char	*temp;
  char **members;
  char *s;

  adr2=adr2 + 11;
  adr2 = strtok( adr2," \n\t" );
  if( ( adr2 == NULL ) || ( *adr2 == 0 ) ) {
    if( *list_name == 0 || control_mode )	/* List name not yet set */
	longjmp( finish,
	    final_status == FINISH_OK ? FINISH_OK : BAD_UNSUBSCRIBE );
  } else
  if( control_mode )
	strcpy( list_name,adr2 );
  read_config( list_name,0 );
  Log( 2,1,"unsubscribe request on list %s for %s",list_name,addr );
  if( !subscribe ) {
    Log( 2,1,"list %s is disabled",list_name );
    longjmp( finish, 
	final_status == FINISH_OK ? FINISH_OK : LIST_DISABLED );
  }
  sprintf( fname, "%s/%s.members", PATH, list_name );
  member = file_open( fname,"r" );
  while((s = getline(member))) if(*s) numlines++;
  rewind(member);
  members=(char **)malloc(numlines * sizeof(char *));
  len=0;
  while((temp=getline(member))!=NULL) {
    if(strstr(temp, addr)==NULL) {
      members[len]=temp;
      len++;
    } else
	success = 1;	/* Address found, remove from list */
  }
  fclose( member );
  if( !success ) {
    Log( 2,1,"address %s not on list %s",addr,list_name );
    longjmp( finish,
	final_status == FINISH_OK ? FINISH_OK : NOT_ON_LIST );
  }
  member = file_open( fname,"w+" );
  for( x = 0 ; x < len ; ++x )
    if (members[x]!=NULL)
	fprintf(member, "%s\n", members[x]);
  fclose( member );
  longjmp( finish,final_status );
}

/* Housekeeping before termination. Generally sends out
   confirmation mails. Returns the new error code */
int housekeep( int errcode ) {
  char *new_subject;
  
  new_subject = subject == NULL ? NULL : subject+9;
  switch( errcode ) {
    case BAD_SUBSCRIBE:
	send_reply( addr,"bad_subscribe",new_subject );
	return FINISH_OK;

    case ALREADY_ON_LIST:
	send_reply( addr,"already_on_list",new_subject );
	return FINISH_OK;

    case LIST_DISABLED:
	send_reply( addr,"list_disabled",new_subject );
	return FINISH_OK;

    case NOT_ALLOWED:
	send_reply( addr,"not_allowed",new_subject );
	return FINISH_OK;

    case SEND_SUBS_MESSAGE:
	send_reply( addr,"subs_message",new_subject );
	return FINISH_OK;

    case BAD_UNSUBSCRIBE:
	send_reply( addr,"bad_unsubscribe",new_subject );
	return FINISH_OK;

    case SEND_UNSUBS_MESSAGE:
	send_reply( addr,"unsubs_message",new_subject );
	return FINISH_OK;

    case HELP_MESSAGE:
	send_reply( addr,"help",new_subject );
	return FINISH_OK;

    case NO_LIST:
	send_reply( addr,"no_list",new_subject );
	return FINISH_OK;
	
    case NOT_ON_LIST:
	send_reply( addr,"not_on_list",new_subject );
	return FINISH_OK;
	
    case SEND_CONF_MESSAGE:
    	send_reply( addr,"subs_confirm",SubsConfID );
    	return FINISH_OK;
    	
    case LIST_CLOSED:
    	send_reply( addr,"list_closed",new_subject );
    	return FINISH_OK;
  }
  return errcode;
}

/* Logs mail loop into the log file */
void log_mailloop() {
  FILE *tempfile;
  char	*p;

  if( LogMailLoop ) {
    tempfile = file_open( TempFileName,"r" );
    fseek( tempfile,0,SEEK_SET );
    while( fgets( work,N_BUFSIZE,tempfile ) != NULL ) {
      if( ( p = strchr( work,'\n' ) ) != NULL )
		    *p = 0;
      Log( 0,0,"MAILLOOP: %s",work );
    }
    fclose( tempfile );
  }
}
   
int main( int argc, char *argv[] ) {

  char zero_char = 0;
  char *s,sep;
  FILE *member,*tempfile,*tempfile2;
  char *adr,*p,*p1,*p2;
  char *adr2=NULL;
  char *dateaddr = NULL;
  char *fromcur=NULL;
  char buf[N_BUFSIZE];
  char **members,**command_array;
  int	fds[2],fc,newpid;
  int len,i,close_out;
  int numlines=0;
  int pass=1;
  int x=0;
  int numread=0;
  int xsender_found;
  int xnonews_found;
  char	addr_buf[N_BUFSIZE];
  char	sfa_buf[N_BUFSIZE];

  init_commonvars();
/* What an ugly solution, I am really ashamed :-) We return here
   at the end of the program to perform some housekeeping before
   quitting ... Sorry but the original program was full of exit()s */
	if( x = setjmp( finish ) ) {
	  x = housekeep( x );
	  if( config != NULL )
		fclose( config );	/* See remark at read_config */
          unlink( TempFileName );
          unlink( TempFileName2 );
          exit( x - 1 );
	}

	*list_name = 0;
	if( argc == 2 ) {
	  strcpy( list_name,argv[1] );
	  control_mode = 0;
	}
	else {
/* control.config may be necessary for list commands */
	  strcpy( list_name,"control" );
	  control_mode = 1;
	}
	read_config( list_name,1 );
/* Opening temporary file for the message */
        sprintf( TempFileName,"%s/nlist%i.tmp",PATH,getpid() );
        strcpy( TempFileName2,TempFileName );
        strcat( TempFileName2,"-2" );
	tempfile = file_open( TempFileName,"w+" );
	xsender_found = 0;
	xnonews_found = 0;
	while( ( adr=getline( stdin ) ) != NULL ) {
loop:
	  if( adr == NULL )
	  	break;
/* Empty line - end of header section */
	  if( *adr == 0 )
	  	break;
	  if( !strncasecmp( adr,"Content-Type: ",14 ) ) {
/* Content-type may be multi-line header with many attributes like
   boundary in MIME headers. We compile the header into one line. */
            adr = CompileMultiLineHeader( adr,content_type,2*N_BUFSIZE,tempfile );
/* Extract the boundary string */
	    if( ( p = strcasestr( content_type,"boundary=" ) ) != NULL ) {
	      p += 9;
	      getWord( p,boundary,128,NULL );
	    }
	    goto loop;  
	  }
	  if( !strncasecmp(adr, "Subject: ", 9 ) ) {
	    adr = CompileMultiLineHeader( adr,orig_subject_string,1000,tempfile );
	    subject = orig_subject_string;
/* That may be vacation auto reply */
	    if( strcasestr( subject,"Auto-reply" ) != NULL )
		xsender_found = 1;
	    goto loop;
	  }
	  if( !strncasecmp( adr, "Content-Transfer-Encoding: ",27 ) ) {
	    content_encoding = adr;
	  }
	  if( !strncasecmp( adr,"Received: ",10 ) ) {
/* We don't really collect the header, just discard it */
	    adr = CompileMultiLineHeader( adr,work,N_BUFSIZE,NULL );
	    goto loop;
	  }	  
	  if( !strncasecmp( adr,"Date: ",6 ) )
		dateaddr = adr;
	  if (!strncasecmp(adr, "From ", 5)) {
	    addr = strtok( adr+5," " );
	    if( sender_full_address == NULL )
		sender_full_address = addr;
	  }
	  if (!strncasecmp(adr, "From: ", 6)) {
	    fromcur=adr;
/* Use address from the From: field if any, else use
   the address in the From field */
	    sender_full_address = adr+6;
	  }
/* If there is an X-NoNews header in the letter, the message
   was sent by nlistd and must not be relayed back to news.
   We just remember it now. */
          if( !strncasecmp( adr,"X-NoNews:",9 ) ) {
            xnonews_found = 1;
          } 
/* If there's an X-Sender: nlist header in the letter, it came
   from nlist and somehow we got into a mail loop. We swallow
   the letter but this time we just signal it in a flag.
   The header checking is a bit overcomplicated as some
   braindead mail servers send back headers without space
   after the colon */
	  if( !strncasecmp( adr,"X-Sender:",9 ) &&
	      ( strstr( adr,"nlist" ) != NULL ) )
	      xsender_found = 1;
/* Set the xsender_found flag if auto-submitted header was found,
   then it is an automatic reply from somewhere that should not
   be processed. */
          if( !strncasecmp( adr,"Auto-Submitted:",15 ) )
	      xsender_found = 1;
	  fprintf( tempfile,"%s\n",adr );
	}
	fprintf( tempfile,"\n" );
/* Now saving the content of the letter */
	ContentPointer = ftell( tempfile );
	while((numread = fread(buf, 1, N_BUFSIZE, stdin)))
	    fwrite( buf,1,numread,tempfile );
/* Now looking for X-Sender: nlist strings in the message
   body. Most mailer daemons quote the full incoming
   message along with its headers when generating error
   letter */
	fseek( tempfile,ContentPointer,SEEK_SET );
	while( fgets( work,N_BUFSIZE,tempfile ) != NULL ) {
	  if( !strncasecmp( work,"X-Sender:",9 ) &&
	      ( strstr( work,"nlist" ) != NULL ) )
		xsender_found = 1;
	  if( !strncasecmp( work,"Auto-Submitted:",15 ) )
	  	xsender_found = 1;
	}
	if( LogLevel > 3 ) {
	  Log( 3,1,"letter arrived" );
	  fseek( tempfile,0,SEEK_SET );
	  while( fgets( work,N_BUFSIZE,tempfile ) != NULL ) {
	    if( ( p = strchr( work,'\n' ) ) != NULL )
		    *p = 0;
	    Log( 3,0,"L: %s",work );
	  }
	}
	fclose( tempfile );
/* It's not OK at all, the message has no From field. We swallow
   it silently */
	if( ( addr == NULL ) || ( *addr == 0 ) )
		longjmp( finish,FINISH_OK );
	if( AutoUnsubscribe &&
	    ( ( adr2 = handle_autounsubscribe( TempFileName,ContentPointer ) ) != NULL ) ) {
	    if( *adr2 == 1 ) {
		Log( 2,1,"%s had non-fatal delivery errors",adr2+1 );
		longjmp( finish,FINISH_OK );
	    } else {
	      Log( 2,1,"%s unsubscribed because this address no longer exists",adr2 );
	      handle_unsubscribe( "unsubscribe", adr2 ,FINISH_OK );
	    }	
        }
/* The message had X-Sender: nlist header, we have a mail loop
   here. Delete the message. */
        if( xsender_found ) {
	  Log( 2,1,"mail loop detected from %s",addr );
          read_config( list_name,0 );
	  log_mailloop();
	  longjmp( finish,FINISH_OK );
	}
/* Analyzing whether it is a list command */
	adr2 = subject;
/* Must have subject for that */
	if( ( adr2 != NULL ) &&
	    ( ( adr2=strchr(adr2, ' ') ) != NULL ) ) {
	  ++adr2;
	  if( strcasestr( adr2, "unsubscribe" ) != NULL )
		handle_unsubscribe( adr2,addr,SEND_UNSUBS_MESSAGE );
	  if( strcasestr( adr2, "subscribe" ) != NULL )
		handle_subscribe( adr2,FALSE );
	  if( strcasestr( adr2,"help" ) != NULL ) {
	    Log( 2,1,"help request for %s",addr );
	    longjmp( finish,HELP_MESSAGE );
	  }
/* Check the subject for subscription confirmation replies if it is allowed */
          if( SubsConfirmation && ( ( p = strcasestr( adr2,"confirm " ) ) != NULL ) ) {
            p += 8;
            s = buf;
/* 16 hexadecimal digits must follow the confirmation subject */
            for( i = 0 ; i < 16 ; ++i ) {
              if( !isxdigit( *p ) )
              		goto endsubject;
              *(s++) = *(p++);
            }
            *s = 0;
            Log( 4,1,"Subscription confirmation request from %s with passcode %s",
            		addr,buf ); 
/* If the passcode is OK, do subscribe else swallow the letter silently */
            if( UpdateConfirmationFile( addr,buf ) )
			handle_subscribe( NULL, TRUE );
	    longjmp( finish, FINISH_OK );
          }
	}
endsubject:
/* Let's check now if there are any control commands in the message body */
	tempfile = file_open( TempFileName,"r" );
	fseek( tempfile,ContentPointer,SEEK_SET );
	while( fgets( work,N_BUFSIZE,tempfile ) != NULL ) {
/* Control command is exactly one word in the line */
          p1 = strtok( work," \t\n\r" );
          if( p1 == NULL )
          	continue;
          p2 = strtok( NULL," \t\n\r" );
          if( p2 != NULL )
          	continue;
	  if( !strcasecmp( p1, "unsubscribe" ) )
		handle_unsubscribe( work,addr,SEND_UNSUBS_MESSAGE );
	  if( !strcasecmp( p1, "subscribe" ) )
		handle_subscribe( work,FALSE );
	  if( !strcasecmp( p1,"help" ) ) {
	    Log( 2,1,"help request for %s",addr );
	    longjmp( finish,HELP_MESSAGE );
	  }	  
	}
        fclose( tempfile );
	if( argc == 2) {		/* Called with list argument */
	  read_config( list_name,0 );
/* Creating new subject into subject_string */
	  *subject_string = 0;
	  if( subject ) {
	    adr2 = strlen( subject ) >= 9 ? subject+9 : &zero_char;
	    if( *list_title != 0 )
              strcpy( subject_string,
	        process_list_title( adr2,list_title ) );
	    if( *subject_string == 0 )
		strcpy( subject_string,adr2 );
	  } else
	  	strcpy( subject_string,list_title );
	  sprintf(fname, "%s/%s.members", PATH, list_name);
	  member = file_open( fname,"r" );
/* Swallow the message silently if error message; should be
   processed ! */
	  if( ( addr != NULL ) &&
	      ( ( strncasecmp( addr,"mailer-daemon",13 ) == 0 ) ||
	        ( strncasecmp( addr,"postmaster",10 ) == 0 ) ) ) {
	    log_mailloop();
	    Log( 2,1,"automatic reply from %s swallowed",addr );
	    longjmp( finish, FINISH_OK );
	  }
/* Check for barred extensions if the feature is enabled */
	  if( *filter_extensions && CheckForbiddenExtensions( TempFileName ) )
		longjmp( finish, FINISH_OK );
/* spam filtering */
	  if( !spamfilt( TempFileName ) ) {
	    Log( 2,1,"letter from %s is filtered out by the spam filter",
			addr );
	    longjmp( finish,FINISH_OK );
	  }
	  if( closed ) {	/* Check membership if the list is closed */
	    pass = 0;
	    x = strlen( addr );
	    while( ( fscanf(member, "%s", work ) ) !=EOF ) {
	      if( strncasecmp( work, addr ,x ) == 0 ) {
	        pass = 1;
	        break;
	      }
	    }
	    if( !pass ) {
/* if there is a passphrase defined */
	      close_out = passphrase[0][0] != 0 ?  
	      			!find_passphrase( TempFileName ) : 1;
	      if( close_out ) {
	        Log( 2,1,"%s is not member of list %s and the list is closed",
	              addr,list_name );
	        longjmp( finish,LIST_CLOSED );
	      }
	    }
	  }
	  fclose( member );
	  sprintf( fname, "%s/%s.members", PATH, list_name);
	  member = file_open( fname,"r" );
	  while( ( s = getline( member ) ) ) if( *s ) numlines++;
	  rewind( member );
/* Now creating parameter array for the listarchiver subprocess */
	  members=(char **)malloc( ( numlines+1 ) * sizeof(char *));
	  for( len = 0 ; len < numlines ; ++len )
		members[len] = getline(member);
	  members[len] = NULL;
	  
/* if archive is switched on, letter is piped into the listarchiver
   utility */
	  tempfile = file_open( TempFileName,"r" );
	  if( *archive != 0 ) {	/* Archiving switched on for this list */
	    command_array = (char **)malloc( 10 * sizeof( char * ) );
	    i = 0;
	    command_array[i++] = "listarchiver";
	    if( *monthindex_name != 0 ) {
	      command_array[i++] = "-i";
	      command_array[i++] = monthindex_name;
	    }
	    if( garble_flag )
	      command_array[i++] = "-g";
	    command_array[i++] = archive;
	    command_array[i++] = NULL;
	    if(pipe(fds)==-1)
	  		ehandle(1, "pipe to listarchiver\n");
	    fc = fcntl(fds[1], F_GETFL, 0);
	    fcntl(fds[1], F_SETFL, fc & ~O_NONBLOCK);
	    newpid = fork();
            if( newpid ) {
              close( fds[0] );
	      if (fromcur) {
	        x=strlen(fromcur);
	        write(fds[1], fromcur, x );
	        write(fds[1], "\n", 1);
	      }
	      if( subject != NULL ) {
	        x=strlen( subject );
	        write(fds[1], subject, x );
	        write(fds[1], "\n", 1);
	      }
	      if( dateaddr ) {
	        x=strlen( dateaddr );
	        write(fds[1], dateaddr, x );
	        write(fds[1], "\n", 1);
	      }
	      if( *content_type != 0 ) {
	        x=strlen( content_type );
	        write(fds[1], content_type, x );
	        write(fds[1], "\n", 1);
	      }
	      if( content_encoding != NULL ) {
	        x = strlen( content_encoding );
	        write(fds[1], content_encoding, x );
	        write(fds[1], "\n", 1);
	      }
	      write(fds[1], "\n", 1);
	      fseek( tempfile,ContentPointer,SEEK_SET );
	      while((numread = fread(buf, 1, N_BUFSIZE-1, tempfile)))
		write(fds[1], buf, numread);
	      close(fds[1]);
 	    } else {
 	      close( fds[1] );
	      close(0);
	      dup2( fds[0] , 0);
	      close( fds[0] );
	      sprintf( work,"%s/listarchiver",PATH );
	      if(execv(work, command_array )==-1)
			ehandle(1, "Unable to execute %s\n",work);
	    }
            free( command_array );
	  }
/* If we post to Usenet and we use senderbanner and there is address garbling, 
   we create a secondary tempfile because the e-mail address in the Usenet message
   will be different */
          if( ( *nntp_newsgroup != 0 ) && !xnonews_found &&
              sender_banner && garble_flag ) {
            rewind( tempfile );
            tempfile2 = file_open( TempFileName2,"w" );
            while( fgets( work,N_BUFSIZE,tempfile ) != NULL )
            	fputs( work,tempfile2 );
            fclose( tempfile2 );           
          }
/* We close and reopen the temporary file as insert_senderbanner may
   recreate it */
	  fclose( tempfile );
	  if( sender_banner )
	  	insert_senderbanner( TempFileName );
	  tempfile = file_open( TempFileName,"r" );
/* Sending the message one by one */
	  for( len = 0 ; len < numlines ; ++len ) {
	    if( members[len] == NULL )
		continue;		/* Paranoia ... :-) */
	    Log( 3,1,"sending letter to %s",members[len] );
	    if( open_smtp_session( smtp_server,addr,members[len] ) ) {
	      close_socket();
	      continue;
	    }
/* Sending headers now */
	    sprintf( buf,"Subject: %s\n",subject_string );
	    write_smtp_line( buf );
	    send_list_headers( members[len] );	    
	    rewind( tempfile );
/* Now send out message headers that nlist doesn't handle.
   These may be MIME content type and content length
   nessages that are necessary to handle attachments */
	    do {
	      if( fgets( buf, N_BUFSIZE, tempfile ) == NULL )
	      	break;
	      if( *buf == '\n' )
	      	break;
	      if( ( strncasecmp( buf,"From:", 5 ) == 0 ) ||
	          ( strncasecmp( buf,"To:", 3 ) == 0 ) ||
	          ( strncasecmp( buf,"Subject:",8 ) == 0 ) ||
	          ( strncasecmp( buf,"Reply-To:", 9 ) == 0 ) ||
	          ( strncasecmp( buf,"X-Sender:",9 ) == 0 ) ||
	          ( strncasecmp( buf,"Errors-To:",10) == 0 ) ||
	          ( strncasecmp( buf,"Date:",5 ) == 0 ) ||
	          ( strncasecmp( buf,"From",4 ) == 0 ) ||
	          ( strncasecmp( buf,"Sender:",7 ) == 0 ) ||
	          ( strncasecmp( buf,"Delivered-To:",13 ) == 0 ) ||
	          ( strncasecmp( buf,"Return-Receipt-To:",18 ) == 0 ) ||
	          ( strncasecmp( buf,"X-NoNews:",9 ) == 0 ) ||
	          ( strncasecmp( buf,"Disposition-Notification-To:",28 ) == 0 ) ||
	          ( strncasecmp( buf,"NNTP-Posting-Host",17 ) == 0 )
	         )
	          continue;
	      write_smtp_line( buf );
	    } while( 1 );
/* Now comes the message body */
	    write_smtp_line( "\n" );
	    fseek( tempfile,ContentPointer,SEEK_SET );
	    while( fgets( buf, N_BUFSIZE, tempfile ) != NULL )
		write_smtp_line( buf );
	    close_smtp_session();
          }	/* for( len == 0 ) */
	  free( members );
          fclose( tempfile );
/* Post to Usenet if requested */
	  if( ( *nntp_newsgroup != 0 ) && !xnonews_found ) {
	    strcpy( addr_buf, addr );
	    strcpy( sfa_buf, sender_full_address );
	    addr = addr_buf;
	    sender_full_address = sfa_buf;
	    if( garble_flag ) {
	      garble_email( addr );
	      garble_email( sender_full_address );
	    }
	    if( sender_banner && garble_flag ) {
	      s = TempFileName2;
	      insert_senderbanner( TempFileName2 );
	    } else
	    	s = TempFileName;
	    if( nntp_post( addr,s ) && ( *nntp_spooldir != 0 ) )
	    		spool_article( s );
	    if( ( *nntp_newsgroup != 0 ) && ( *nntp_spooldir != 0 ) )
 			scan_spool_dir();
 	  }
	}	/* if( argc == 2 ) */
	longjmp( finish, FINISH_OK );
	return( 0 );		/* So that compiler doesn't complain */
}
